@@ -272,19 +272,49 @@ get_slider_position (GtkSourceMap *map,
 	slider_area->y -= us_visible_rect.y;
 }
 
+static void
+gtk_source_map_allocate_slider (GtkSourceMap *map)
+{
+	GtkSourceMapPrivate *priv = gtk_source_map_get_instance_private (map);
+	GdkRectangle area;
+	int width, height;
+	int min, nat;
+
+	g_assert (GTK_SOURCE_IS_MAP (map));
+
+	width = gtk_widget_get_width (GTK_WIDGET (map));
+	height = gtk_widget_get_height (GTK_WIDGET (map));
+
+	if (width == 0 || height == 0)
+		return;
+
+	get_slider_position (map, width, height, &area);
+	gtk_widget_measure (GTK_WIDGET (priv->slider),
+	                    GTK_ORIENTATION_VERTICAL,
+	                    width, &min, &nat, NULL, NULL);
+	area.height = MAX (nat, area.height);
+	gtk_widget_size_allocate (GTK_WIDGET (priv->slider), &area, -1);
+}
+
 static void
 gtk_source_map_rebuild_css (GtkSourceMap *map)
 {
 	GtkSourceMapPrivate *priv;
 	GtkSourceStyleScheme *style_scheme;
 	GtkSourceStyle *style = NULL;
+	GtkSourceStyle *text = NULL;
 	GtkTextBuffer *buffer;
+	GdkRGBA real_bg;
 	const char *color;
 	gboolean use_fg = FALSE;
 	GString *gstr;
 	char *background = NULL;
 	char *foreground = NULL;
 
+	G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+	GtkStyleContext *style_context;
+	G_GNUC_END_IGNORE_DEPRECATIONS
+
 	priv = gtk_source_map_get_instance_private (map);
 
 	if (priv->view == NULL)
@@ -332,8 +362,29 @@ gtk_source_map_rebuild_css (GtkSourceMap *map)
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (priv->view));
 	style_scheme = gtk_source_buffer_get_style_scheme (GTK_SOURCE_BUFFER (buffer));
 
+	G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+	style_context = gtk_widget_get_style_context (GTK_WIDGET (map));
+	if (!gtk_style_context_lookup_color (style_context, "view_bg_color", &real_bg))
+		memset (&real_bg, 0, sizeof (real_bg));
+	G_GNUC_END_IGNORE_DEPRECATIONS
+
 	if (style_scheme != NULL)
 	{
+		if ((text = gtk_source_style_scheme_get_style (style_scheme, "text")))
+		{
+			char *str;
+
+			g_object_get (text,
+				      "background", &str,
+				      NULL);
+
+			if (str != NULL)
+			{
+				gdk_rgba_parse (&real_bg, str);
+				g_free (str);
+			}
+		}
+
 		if (!(style = gtk_source_style_scheme_get_style (style_scheme, "map-overlay")) &&
 		    !(style = gtk_source_style_scheme_get_style (style_scheme, "selection")))
 		{
@@ -417,18 +468,40 @@ gtk_source_map_rebuild_css (GtkSourceMap *map)
 
 	if (color != NULL)
 	{
+		GdkRGBA to_mix;
+		GdkRGBA normal;
+		GdkRGBA hover;
+		GdkRGBA active;
+		char *normal_str;
+		char *hover_str;
+		char *active_str;
+
+		gdk_rgba_parse (&to_mix, color);
+
+		premix_colors (&normal, &to_mix, &real_bg, real_bg.alpha > 0., .25);
+		premix_colors (&hover, &to_mix, &real_bg, real_bg.alpha > 0., .35);
+		premix_colors (&active, &to_mix, &real_bg, real_bg.alpha > 0., .5);
+
+		normal_str = gdk_rgba_to_string (&normal);
+		hover_str = gdk_rgba_to_string (&hover);
+		active_str = gdk_rgba_to_string (&active);
+
 		g_string_append_printf (gstr,
 		                        "slider {"
-		                        " background-color: alpha(%s,.25);"
+		                        " background-color: %s;"
 		                        " transition-duration: 300ms;"
 		                        "}\n"
 		                        "slider:hover {"
-		                        " background-color: alpha(%s,.35);"
+		                        " background-color: %s;"
 		                        "}\n"
 		                        "slider.dragging:hover {"
-		                        " background-color: alpha(%s,.5);"
+		                        " background-color: %s;"
 		                        "}\n",
-		                        color, color, color);
+					normal_str, hover_str, active_str);
+
+		g_free (normal_str);
+		g_free (hover_str);
+		g_free (active_str);
 	}
 
 	g_free (background);
@@ -481,7 +554,7 @@ update_child_vadjustment (GtkSourceMap *map)
 
 	gtk_adjustment_set_value (child_vadj, new_value);
 
-	gtk_widget_queue_allocate (GTK_WIDGET (map));
+	gtk_source_map_allocate_slider (map);
 }
 
 static gboolean
@@ -673,6 +746,11 @@ gtk_source_map_measure (GtkWidget      *widget,
 
 	priv = gtk_source_map_get_instance_private (map);
 
+	if (priv->view == NULL)
+	{
+		return;
+	}
+
 	if (orientation == GTK_ORIENTATION_HORIZONTAL)
 	{
 		if (priv->font_desc == NULL)
@@ -1198,20 +1276,12 @@ gtk_source_map_size_allocate (GtkWidget *widget,
                               int        baseline)
 {
 	GtkSourceMap *map = (GtkSourceMap *)widget;
-	GtkSourceMapPrivate *priv = gtk_source_map_get_instance_private (map);
-	GdkRectangle area;
-	int min, nat;
 
 	g_assert (GTK_SOURCE_IS_MAP (map));
 
 	GTK_WIDGET_CLASS (gtk_source_map_parent_class)->size_allocate (widget, width, height, baseline);
 
-	get_slider_position (map, width, height, &area);
-	gtk_widget_measure (GTK_WIDGET (priv->slider),
-	                    GTK_ORIENTATION_VERTICAL,
-			    width, &min, &nat, NULL, NULL);
-	area.height = MAX (nat, area.height);
-	gtk_widget_size_allocate (GTK_WIDGET (priv->slider), &area, -1);
+	gtk_source_map_allocate_slider (map);
 }
 
 static void
@@ -1224,9 +1294,12 @@ gtk_source_map_snapshot (GtkWidget   *widget,
 	g_assert (GTK_IS_WIDGET (widget));
 	g_assert (GTK_IS_SNAPSHOT (snapshot));
 
-	GTK_WIDGET_CLASS (gtk_source_map_parent_class)->snapshot (widget, snapshot);
-
+	/* Render the slider behind the contents so they are more legible and
+	 * we can avoid an RGBA blend on top of the contents.
+	 */
 	gtk_widget_snapshot_child (GTK_WIDGET (map), GTK_WIDGET (priv->slider), snapshot);
+
+	GTK_WIDGET_CLASS (gtk_source_map_parent_class)->snapshot (widget, snapshot);
 }
 
 static void
@@ -1354,6 +1427,7 @@ gtk_source_map_init (GtkSourceMap *map)
 	gtk_widget_add_controller (GTK_WIDGET (map), g_steal_pointer (&scroll));
 
 	press = GTK_EVENT_CONTROLLER (gtk_gesture_click_new ());
+	gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (press), 0);
 	g_signal_connect_swapped (press,
 	                          "pressed",
 	                          G_CALLBACK (gtk_source_map_click_pressed),
